#
#   module tank.core.lambda
#


def first(subscriptable):
    return subscriptable[0]


def second(subscriptable):
    return subscriptable[1]


def third(subscriptable):
    return subscriptable[2]
